package cn.com.infosec.jce.oscca;

import cn.com.infosec.asn1.ASN1Object;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DERSequence;
import cn.com.infosec.math.ec.ECConstants;
import cn.com.infosec.math.ec.ECCurve;
import cn.com.infosec.math.ec.ECFieldElement;
import cn.com.infosec.math.ec.ECPoint;
import cn.com.infosec.util.BigIntegers;
import java.io.IOException;
import java.math.BigInteger;

public class SM2Util {
   public static final int SM2_SIZE = 32;
   public static final BigInteger gmp;
   public static final BigInteger gma;
   public static final BigInteger gmb;
   public static final ECCurve gmec256;
   public static final BigInteger gmgx;
   public static final BigInteger gmgy;
   public static final ECPoint gmg;
   public static final BigInteger gmn;
   static ECFieldElement ONE;
   static ECFieldElement TWO;
   static ECFieldElement FOUR;
   static ECFieldElement THREE;
   static ECFieldElement EIGHT;
   static ECFieldElement A;

   static {
      BigInteger var0;
      BigInteger var10000 = var0 = new BigInteger;
      var10000.<init>("FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FFFFFFFFFFFFFFFF", 16);
      gmp = var10000;
      BigInteger var1;
      var10000 = var1 = new BigInteger;
      var10000.<init>("FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FFFFFFFFFFFFFFFC", 16);
      gma = var10000;
      BigInteger var2;
      var10000 = var2 = new BigInteger;
      var10000.<init>("28E9FA9E9D9F5E344D5A9E4BCF6509A7F39789F515AB8F92DDBCBD414D940E93", 16);
      gmb = var10000;
      gmg = (gmec256 = new ECCurve.Fp(var0, var1, var2)).createPoint(gmgx = new BigInteger("32C4AE2C1F1981195F9904466A39C9948FE30BBFF2660BE1715A4589334C74C7", 16), gmgy = new BigInteger("BC3736A2F4F6779C59BDCEE36B692153D0A9877CC62A474002DF32E52139F0A0", 16), false);
      gmn = new BigInteger("FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFF7203DF6B21C6052B53BBF40939D54123", 16);
      ONE = new ECFieldElement.Fp(var0, ECConstants.ONE);
      TWO = new ECFieldElement.Fp(var0, ECConstants.TWO);
      FOUR = new ECFieldElement.Fp(var0, BigInteger.valueOf(4L));
      THREE = new ECFieldElement.Fp(var0, BigInteger.valueOf(3L));
      EIGHT = new ECFieldElement.Fp(var0, BigInteger.valueOf(8L));
      A = new ECFieldElement.Fp(var0, var1);
   }

   public static byte[] getZA(byte[] id, byte[] pubkey) {
      byte[] var10001 = id;
      byte[] var10002 = id;
      byte[] var10003 = id;
      byte[] var10004 = id;
      id = new byte[id.length + 2 + 192];
      System.arraycopy(getENTLa(var10004), 0, id, 0, 2);
      int var2 = var10003.length;
      System.arraycopy(var10002, 0, id, 2, var2);
      var2 = 2 + var10001.length;
      System.arraycopy(gma.toByteArray(), 0, id, var2, 32);
      var2 += 32;
      System.arraycopy(gmb.toByteArray(), 0, id, var2, 32);
      var2 += 32;
      System.arraycopy(gmgx.toByteArray(), 0, id, var2, 32);
      var2 += 32;
      System.arraycopy(gmgy.toByteArray(), 0, id, var2, 32);
      var2 += 32;
      if (pubkey.length == 64) {
         byte[] var10000 = pubkey;
         int pubkey1 = pubkey.length;
         System.arraycopy(var10000, 0, id, var2, pubkey1);
      } else if (pubkey.length == 65) {
         System.arraycopy(pubkey, 1, id, var2, 64);
      } else if (pubkey.length == 33) {
         System.arraycopy(pubkey, 1, id, var2, 32);
         var2 += 32;
         byte var3 = pubkey[0];
         pubkey[0] = 0;
         System.arraycopy(BigIntegers.asUnsignedByteArray(getY(new BigInteger(1, pubkey), var3)), 0, id, var2, 32);
      }

      return SM3.SM3Digest(id);
   }

   private static BigInteger getBeta(BigInteger x) {
      BigInteger var1 = null;
      BigInteger var2;
      var2 = new BigInteger.<init>("3");
      BigInteger var3;
      BigInteger var10000 = var3 = gmp;
      x = x.modPow(var2, var3).add(gma.multiply(x)).add(gmb).mod(var3);
      BigInteger[] var5;
      switch ((var5 = var10000.divideAndRemainder(new BigInteger("8")))[1].intValue()) {
         case 5:
            int var4;
            if ((var4 = x.modPow(var5[0].add(var5[0]).add(var1 = BigInteger.ONE), var3).intValue()) != -1) {
               if (var4 != 1) {
                  return null;
               }

               var1 = x.modPow(var5[0].add(var1), var3);
            } else {
               var10000 = x.add(x);
               var1 = var10000.multiply(var10000.add(var10000).modPow(var5[0], var3));
            }
            break;
         case 6:
         case 7:
            var10000 = var1 = x.modPow(var5[0].add(var5[0]).add(BigInteger.ONE), var3);
            if (var10000.multiply(var10000).mod(var3).compareTo(x) != 0) {
               return null;
            }
      }

      return var1;
   }

   static BigInteger getY(BigInteger x, int ypb) {
      if ((x = getBeta(x)) == null) {
         return null;
      } else {
         byte[] var10000 = BigIntegers.asUnsignedByteArray(x);
         return (var10000[var10000.length - 1] & 255) == ypb - 2 ? x : gmp.add(x.negate());
      }
   }

   public static byte[] point2octect(ECPoint p) {
      byte[] var1;
      (var1 = new byte[65])[0] = 4;
      byte[] var2;
      if ((var2 = BigIntegers.asUnsignedByteArray(p.getX().toBigInteger())).length > 32) {
         return null;
      } else {
         ECPoint var10000 = p;
         byte[] var10001 = var2;
         int p1 = 33 - var2.length;
         int var5 = var2.length;
         System.arraycopy(var10001, 0, var1, p1, var5);
         byte[] p2;
         if ((p2 = BigIntegers.asUnsignedByteArray(var10000.getY().toBigInteger())).length > 32) {
            return null;
         } else {
            var10001 = p2;
            byte[] var10002 = p2;
            p1 = 65 - p2.length;
            var5 = var10002.length;
            System.arraycopy(var10001, 0, var1, p1, var5);
            return var1;
         }
      }
   }

   public static void point2octect(ECPoint p, byte[] x, byte[] y) {
      System.arraycopy(BigIntegers.asUnsignedByteArray(p.getX().toBigInteger()), 0, x, 0, 32);
      System.arraycopy(BigIntegers.asUnsignedByteArray(p.getY().toBigInteger()), 0, y, 0, 32);
   }

   public static ECPoint octect2point(byte[] pb) {
      byte[] var1 = new byte[32];
      byte[] var2 = new byte[32];
      int var3;
      BigInteger pb1;
      if ((var3 = pb.length) != 33) {
         if (var3 != 64) {
            if (var3 != 65) {
               return null;
            }

            byte var7 = 1;
            byte var4 = 33;
            if (pb[0] != 4) {
               return null;
            }

            System.arraycopy(pb, var7, var1, 0, 32);
            System.arraycopy(pb, var4, var2, 0, 32);
         } else {
            System.arraycopy(pb, 0, var1, 0, 32);
            System.arraycopy(pb, 32, var2, 0, 32);
         }

         ECCurve var8 = gmec256;
         BigInteger var10001 = new BigInteger(1, var1);
         pb1 = new BigInteger.<init>(1, var2);
         return var8.createPoint(var10001, pb1, false);
      } else {
         System.arraycopy(pb, 1, var1, 0, 32);
         BigInteger var6;
         BigInteger var10000 = var6 = new BigInteger;
         var6.<init>(1, var1);
         return (pb1 = getY(var10000, pb[0])) == null ? null : gmec256.createPoint(var6, pb1, false);
      }
   }

   static byte[] getENTLa(byte[] id) {
      byte[] var1;
      byte[] var10000 = var1 = new byte[2];
      int var10002 = id.length * 8;
      var1[1] = (byte)(id.length * 8 & 255);
      var10000[0] = (byte)(var10002 >> 8 & 255);
      return var10000;
   }

   public static byte[] derEncode(BigInteger r, BigInteger s) {
      IOException var10000;
      label29: {
         boolean var10001;
         DERInteger[] var6;
         try {
            var6 = new DERInteger[2];
         } catch (IOException var5) {
            var10000 = var5;
            var10001 = false;
            break label29;
         }

         DERInteger[] var2;
         DERInteger[] var7 = var2 = var6;
         byte var10002 = 0;

         try {
            var7[var10002] = new DERInteger(r);
         } catch (IOException var4) {
            var10000 = var4;
            var10001 = false;
            break label29;
         }

         byte var8 = 1;

         try {
            var6[var8] = new DERInteger(s);
            return (new DERSequence(var2)).getEncoded("DER");
         } catch (IOException var3) {
            var10000 = var3;
            var10001 = false;
         }
      }

      var10000.printStackTrace();
      return null;
   }

   public static BigInteger[] derDecode(byte[] encoding) {
      IOException var10000;
      label37: {
         ASN1Sequence var7;
         boolean var10001;
         try {
            var7 = (ASN1Sequence)ASN1Object.fromByteArray(encoding);
         } catch (IOException var4) {
            var10000 = var4;
            var10001 = false;
            break label37;
         }

         ASN1Sequence encoding1 = var7;

         BigInteger[] var8;
         try {
            var8 = new BigInteger[2];
         } catch (IOException var3) {
            var10000 = var3;
            var10001 = false;
            break label37;
         }

         BigInteger[] var9 = var8;
         BigInteger[] var10003 = var8;
         ASN1Sequence var10002 = encoding1;
         ASN1Sequence var10004 = encoding1;
         byte encoding2 = 0;

         try {
            var10003[encoding2] = ((DERInteger)var10004.getObjectAt(0)).getValue();
         } catch (IOException var2) {
            var10000 = var2;
            var10001 = false;
            break label37;
         }

         encoding2 = 1;

         try {
            var9[encoding2] = ((DERInteger)var10002.getObjectAt(1)).getValue();
            return var8;
         } catch (IOException var1) {
            var10000 = var1;
            var10001 = false;
         }
      }

      var10000.printStackTrace();
      return null;
   }

   public static byte[] I2OSP(int x) {
      byte[] var1;
      byte[] var10000 = var1 = new byte[4];
      var1[0] = (byte)(x >>> 24);
      var1[1] = (byte)(x >>> 16);
      var1[2] = (byte)(x >>> 8);
      var10000[3] = (byte)x;
      return var10000;
   }
}
